@@ -6,11 +6,11 @@ is.na.bbox = function(x) identical(x, NA_bbox_)
 
 bb_wrap = function(bb) {
 	stopifnot(is.numeric(bb) && length(bb) == 4)
-	structure(bb, names = c("xmin", "ymin", "xmax", "ymax"), class = "bbox")
+	structure(as.double(bb), names = c("xmin", "ymin", "xmax", "ymax"), class = "bbox")
 }
 
 bbox.Set = function(obj, ...) {
-	sel = vapply(obj, function(x) { length(x) && !all(is.na(x)) }, TRUE)
+	sel = !sfc_is_empty(obj)
 	if (! any(sel))
 		NA_bbox_
 	else
@@ -45,7 +45,7 @@ bbox.MtrxSetSetSet = function(obj, ...) {
 #'
 #' Return bounding of a simple feature or simple feature set
 #' @param obj object to compute the bounding box from
-#' @param ... ignored
+#' @param ... for format.bbox, passed on to \link[base]{format} to format individual numbers
 #' @export
 #' @return a numeric vector of length four, with \code{xmin}, \code{ymin}, \code{xmax}
 #' and \code{ymax} values; if \code{obj} is of class \code{sf}, \code{sfc}, \code{Spatial} or \code{Raster}, the object
@@ -129,8 +129,8 @@ st_bbox.CIRCULARSTRING = function(obj, ...) {
 
 #' @export
 print.bbox = function(x, ...) {
-	attr(x, "crs") = NULL
-	print(unclass(x))
+	x = structure(x, crs = NULL, class = NULL)
+	print(set_units(x, attr(x, "units"), mode = "standard"))
 }
 
 compute_bbox = function(obj) {
@@ -160,7 +160,7 @@ st_bbox.Spatial = function(obj, ...) {
 		stop("package sp required, please install it first")
 	bb = sp::bbox(obj)
 	structure(bb_wrap(c(bb[1,1],bb[2,1],bb[1,2],bb[2,2])),
-		crs = st_crs(sp::proj4string(obj)))
+		crs = st_crs(slot(obj, "proj4string")))
 }
 
 #' @name st_bbox
@@ -218,3 +218,11 @@ NA_bbox_ = structure(rep(NA_real_, 4),
 	names = c("xmin", "ymin", "xmax", "ymax"),
 	crs = NA_crs_,
 	class = "bbox")
+
+#' @name st_bbox
+#' @export
+format.bbox = function(x, ...) {
+	p1 = paste0("(",paste0(c(format(x[1], ...), format(x[2], ...)), collapse=","),")")
+	p2 = paste0("(",paste0(c(format(x[3], ...), format(x[4], ...)), collapse=","),")")
+	paste0("(",p1,",",p2,")")
+}
