@@ -20,7 +20,7 @@
 #' # non-default offset:
 #' plot(st_make_grid(sfc, cellsize = .1, square = FALSE, offset = c(0, .05 / (sqrt(3)/2))))
 #' plot(sfc, add = TRUE)
-#' nc = read_sf(system.file("shape/nc.shp", package="sf"))
+#' nc = st_read(system.file("shape/nc.shp", package="sf"))
 #' g = st_make_grid(nc)
 #' plot(g)
 #' plot(st_geometry(nc), add = TRUE)
@@ -84,30 +84,23 @@ st_make_grid = function(x,
 		for (i in 1:nx)
 			for (j in 1:ny)
 				ret[[(j - 1) * nx + i]] = square(xc[i], yc[j], xc[i+1], yc[j+1])
+		st_sfc(ret, crs = crs)
 	} else if (what == "centers") {
-		ret = vector("list", nx * ny)
-		cent = function(x1, y1, x2, y2)
-			st_point(c( (x1+x2)/2, (y1+y2)/2 ))
-		for (i in 1:nx)
-			for (j in 1:ny)
-				ret[[(j - 1) * nx + i]] = cent(xc[i], yc[j], xc[i+1], yc[j+1])
+		e = expand.grid(x = xc[-1] - 0.5 * diff(xc[1:2]), 
+						y = yc[-1] - 0.5 * diff(yc[1:2]), 
+						KEEP.OUT.ATTRS = FALSE)
+		st_geometry(st_as_sf(e, coords = c("x", "y"), crs = crs))
 	} else if (what == "corners") {
-		ret = vector("list", (nx + 1) * (ny + 1))
-		for (i in 1:(nx + 1))
-			for (j in 1:(ny + 1))
-				ret[[(j - 1) * (nx + 1) + i]] = st_point(c(xc[i], yc[j]))
+		e = expand.grid(x = xc, y = yc, KEEP.OUT.ATTRS = FALSE)
+		st_geometry(st_as_sf(e, coords = c("x", "y"), crs = crs))
 	} else
 		stop("unknown value of `what'")
-
-	st_sfc(ret, crs = crs)
 }
 
-
 ### hex grid tesselation that
 ## - covers a bounding box st_bbox(obj)
 ## - contains pt
 ## - has x spacing dx: the shortest distance between x coordinates with identical y coordinate
-## - selects geometries intersecting with obj
 make_hex_grid = function(obj, pt, dx, what, flat_topped = TRUE) {
 
 	dy = sqrt(3) * dx / 2
@@ -157,13 +150,8 @@ make_hex_grid = function(obj, pt, dx, what, flat_topped = TRUE) {
 				st_polygon(list(xy[i_from_x(m),2:1]))
 			}
 
-	ret = if (what == "points")
-			st_sfc(lapply(seq_len(nrow(centers)), function(i) mk_point(centers[i,])), crs = st_crs(bb))
-		else # points:
-			st_sfc(lapply(seq_len(nrow(centers)), function(i) mk_pol(centers[i,])), crs = st_crs(bb))
-
-	if (what == "points" || min(st_dimension(obj)) < 2)
-		ret[obj]
-	else 
-		ret[lengths(st_relate(ret, obj, "2********")) > 0] # part or total overlap
+	if (what == "centers")
+		st_sfc(lapply(seq_len(nrow(centers)), function(i) mk_point(centers[i,])), crs = st_crs(bb))
+	else # points:
+		st_sfc(lapply(seq_len(nrow(centers)), function(i) mk_pol(centers[i,])), crs = st_crs(bb))
 }
