@@ -20,11 +20,12 @@ st_sample = function(x, size, ...) UseMethod("st_sample")
 #' @param warn_if_not_integer logical; if \code{FALSE} then no warning is emitted if \code{size} is not an integer
 #' @param ... passed on to \link[base]{sample} for \code{multipoint} sampling, or to \code{spatstat} functions for spatstat sampling types (see details)
 #' @param type character; indicates the spatial sampling type; one of \code{random}, \code{hexagonal} (triangular really), \code{regular},
-#' or one of the \code{spatstat} methods such as \code{Thomas} for calling \code{spatstat.core::rThomas} (see Details).
+#' or one of the \code{spatstat} methods such as \code{Thomas} for calling \code{spatstat.random::rThomas} (see Details).
 #' @param exact logical; should the length of output be exactly
 #' @param by_polygon logical; for \code{MULTIPOLYGON} geometries, should the effort be split by \code{POLYGON}? See https://github.com/r-spatial/sf/issues/1480
 #' the same as specified by \code{size}? \code{TRUE} by default. Only applies to polygons, and
 #' when \code{type = "random"}.
+#' @param progress logical; if \code{TRUE} show progress bar (only if \code{size} is a vector).
 #' @return an \code{sfc} object containing the sampled \code{POINT} geometries
 #' @details if \code{x} has dimension 2 (polygons) and geographical coordinates (long/lat), uniform random sampling on the sphere is applied, see e.g. \url{http://mathworld.wolfram.com/SpherePointPicking.html}
 #'
@@ -33,7 +34,7 @@ st_sample = function(x, size, ...) UseMethod("st_sample")
 #' As parameter called \code{offset} can be passed to control ("fix") regular or hexagonal sampling: for polygons a length 2 numeric vector (by default: a random point from \code{st_bbox(x)}); for lines use a number like \code{runif(1)}.
 #'
 #' Sampling methods from package \code{spatstat} are interfaced (see examples), and need their own parameters to be set. 
-#' For instance, to use \code{spatstat.core::rThomas()}, set \code{type = "Thomas"}.
+#' For instance, to use \code{spatstat.random::rThomas()}, set \code{type = "Thomas"}.
 #' @examples
 #' nc = st_read(system.file("shape/nc.shp", package="sf"))
 #' p1 = st_sample(nc[1:3, ], 6)
@@ -45,6 +46,7 @@ st_sample = function(x, size, ...) UseMethod("st_sample")
 #' plot(x, axes = TRUE, graticule = TRUE)
 #' if (sf_extSoftVersion()["proj.4"] >= "4.9.0")
 #'   plot(p <- st_sample(x, 1000), add = TRUE)
+#' if (require(lwgeom, quietly = TRUE)) { # for st_segmentize()
 #' x2 = st_transform(st_segmentize(x, 1e4), st_crs("+proj=ortho +lat_0=30 +lon_0=45"))
 #' g = st_transform(st_graticule(), st_crs("+proj=ortho +lat_0=30 +lon_0=45"))
 #' plot(x2, graticule = g)
@@ -52,6 +54,7 @@ st_sample = function(x, size, ...) UseMethod("st_sample")
 #'   p2 = st_transform(p, st_crs("+proj=ortho +lat_0=30 +lon_0=45"))
 #'   plot(p2, add = TRUE)
 #' }
+#' }
 #' x = st_sfc(st_polygon(list(rbind(c(0,0),c(90,0),c(90,10),c(0,90),c(0,0))))) # NOT long/lat:
 #' plot(x)
 #' p_exact = st_sample(x, 1000, exact = TRUE)
@@ -81,9 +84,9 @@ st_sample = function(x, size, ...) UseMethod("st_sample")
 #' st_sample(ls, 80)
 #' plot(st_sample(ls, 80))
 #' # spatstat example:
-#' if (require(spatstat.core)) {
+#' if (require(spatstat.random)) {
 #'  x <- sf::st_sfc(sf::st_polygon(list(rbind(c(0, 0), c(10, 0), c(10, 10), c(0, 0)))))
-#'  # for spatstat.core::rThomas(), set type = "Thomas":
+#'  # for spatstat.random::rThomas(), set type = "Thomas":
 #'  pts <- st_sample(x, kappa = 1, mu = 10, scale = 0.1, type = "Thomas") 
 #' }
 #' @export
@@ -93,13 +96,18 @@ st_sample.sf = function(x, size, ...) st_sample(st_geometry(x), size, ...)
 #' @export
 #' @name st_sample
 st_sample.sfc = function(x, size, ..., type = "random", exact = TRUE, warn_if_not_integer = TRUE,
-		by_polygon = FALSE) {
+		by_polygon = FALSE, progress = FALSE) {
 
 	if (!missing(size) && warn_if_not_integer && any(size %% 1 != 0))
 		warning("size is not an integer")
 	if (!missing(size) && length(size) > 1) { # recurse:
 		size = rep(size, length.out = length(x))
-		ret = lapply(1:length(x), function(i) st_sample(x[i], size[i], type = type, exact = exact, ...))
+		ret = if (progress) {
+				if (!requireNamespace("pbapply", quietly = TRUE))
+					stop("package pbapply required, please install it first")
+				pbapply::pblapply(1:length(x), function(i) st_sample(x[i], size[i], type = type, exact = exact, ...))
+			} else
+				lapply(1:length(x), function(i) st_sample(x[i], size[i], type = type, exact = exact, ...))
 		st_set_crs(do.call(c, ret), st_crs(x))
 	} else {
 		res = switch(max(st_dimension(x)) + 1,
@@ -180,15 +188,16 @@ st_poly_sample = function(x, size, ..., type = "random",
 				} else
 					runif(size, bb[2], bb[4])
 				m = cbind(lon, lat)
+				colnames(m) = NULL
 				st_sfc(lapply(seq_len(nrow(m)), function(i) st_point(m[i,])), crs = st_crs(x))
 			}
 		pts[x] # cut out x from bbox
 	} else { # try to go into spatstat
-		if (!requireNamespace("spatstat.core", quietly = TRUE))
-			stop("package spatstat.core required, please install it (or the full spatstat package) first")
-		spatstat_fun = try(get(paste0("r", type), asNamespace("spatstat.core")), silent = TRUE)
+		if (!requireNamespace("spatstat.random", quietly = TRUE))
+			stop("package spatstat.random required, please install it (or the full spatstat package) first")
+		spatstat_fun = try(get(paste0("r", type), asNamespace("spatstat.random")), silent = TRUE)
 		if (inherits(spatstat_fun, "try-error"))
-			stop(paste0("r", type), " is not an exported function from spatstat.core.")
+			stop(paste0("r", type), " is not an exported function from spatstat.random.")
 		pts = try(spatstat_fun(..., win = spatstat.geom::as.owin(x)), silent = TRUE)
 		if (inherits(pts, "try-error"))
 			stop("The spatstat function ", paste0("r", type),
@@ -205,7 +214,7 @@ st_multipoints_sample = function(x, size, ..., type = "random") {
 	st_sfc(st_multipoint(m[sample(nrow(m), size, ...),]), crs = st_crs(x))
 }
 
-st_ll_sample = function (x, size, ..., type = "random", offset = runif(1)) {
+st_ll_sample = function(x, size, ..., type = "random", offset = runif(1)) {
 	crs = st_crs(x)
 	if (isTRUE(st_is_longlat(x))) {
 		message_longlat("st_sample")
@@ -246,9 +255,10 @@ hex_grid_points = function(obj, pt, dx) {
 	x = seq(xlim[1] - dx, xlim[2] + dx, dx) + offset[1]
 	y = seq(ylim[1] - 2 * dy, ylim[2] + 2 * dy, dy) + offset[2]
 
-	y  <- rep(y, each = length(x))
-	x  <- rep(c(x, x + dx / 2), length.out = length(y))
-	xy = cbind(x, y)[x >= xlim[1] & x <= xlim[2] & y >= ylim[1] & y <= ylim[2], ]
+	y = rep(y, each = length(x))
+	x = rep(c(x, x + dx / 2), length.out = length(y))
+	xy = cbind(x, y)[x >= xlim[1] & x <= xlim[2] & y >= ylim[1] & y <= ylim[2], , drop = FALSE]
+	colnames(xy) = NULL
 	st_sfc(lapply(seq_len(nrow(xy)), function(i) st_point(xy[i,])), crs = st_crs(bb))
 }
 
@@ -259,7 +269,7 @@ st_sample_exact = function(x, size, ..., type, by_polygon) {
 		random_pt_new = st_sample(x, size = diff, ..., type, exact = FALSE, by_polygon = by_polygon)
 		random_pt = c(random_pt, random_pt_new)
 	}
-	if(length(random_pt) > size) {
+	if (length(random_pt) > size) {
 		random_pt = random_pt[1:size]
 	}
 	random_pt
