@@ -29,10 +29,12 @@ st_as_sf = function(x, ...) UseMethod("st_as_sf")
 #' df = st_as_sf(d, wkt = "geom")
 #' d$geom2 = st_sfc(pt1, pt2)
 #' st_as_sf(d) # should warn
-#' data(meuse, package = "sp")
-#' meuse_sf = st_as_sf(meuse, coords = c("x", "y"), crs = 28992, agr = "constant")
-#' meuse_sf[1:3,]
-#' summary(meuse_sf)
+#' if (require(sp, quietly = TRUE)) {
+#'  data(meuse, package = "sp")
+#'  meuse_sf = st_as_sf(meuse, coords = c("x", "y"), crs = 28992, agr = "constant")
+#'  meuse_sf[1:3,]
+#'  summary(meuse_sf)
+#' }
 #' @export
 st_as_sf.data.frame = function(x, ..., agr = NA_agr_, coords, wkt,
 		dim = "XYZ", remove = TRUE, na.fail = TRUE, sf_column_name = NULL) {
@@ -450,7 +452,7 @@ as.data.frame.sf = function(x, ...) {
 
 #' @export
 #' @name st_geometry
-#' @details \code{st_drop_geometry} drops the geometry of its argument, and reclasses it accordingly
+#' @details if \code{x} is of class \code{sf}, \code{st_drop_geometry} drops the geometry of its argument, and reclasses it accordingly; otherwise it does nothing.
 st_drop_geometry = function(x, ...) UseMethod("st_drop_geometry")
 
 #' @export
@@ -459,6 +461,12 @@ st_drop_geometry.sf = function(x, ...) {
 	st_set_geometry(x, NULL)
 }
 
+#' @export
+#' @name st_geometry
+st_drop_geometry.default = function(x, ...) {
+	x
+}
+
 #' transform method for sf objects
 #' 
 #' Can be used to create or modify attribute variables; for transforming geometries see 
