@@ -20,6 +20,7 @@ test_that("we can subset sf objects", {
 })
 
 test_that("we can create points sf from data.frame", {
+  skip_if_not_installed("sp")
   data(meuse, package = "sp") # load data.frame from sp
   meuse_sf = st_as_sf(meuse, coords = c("x", "y"), crs = 28992)
   meuse_sf[1:5,]
@@ -63,6 +64,7 @@ test_that("rbind/cbind work", {
 })
 
 test_that("st_as_sf bulk points work", {
+  skip_if_not_installed("sp")
   data(meuse, package = "sp") # load data.frame from sp
   x <- meuse
   meuse_sf = st_as_sf(x, coords = c("x", "y"), crs = 28992)
@@ -82,6 +84,7 @@ test_that("st_as_sf bulk points work", {
 })
 
 test_that("transform work", {
+  skip_if_not_installed("sp")
   data(meuse, package = "sp")
   x  = st_as_sf(meuse, coords = c("x", "y"), crs = 28992)
   x2 = transform(x, elev2 = elev^2, lead_zinc = lead/zinc)
@@ -90,4 +93,11 @@ test_that("transform work", {
   expect_identical(st_bbox(x), st_bbox(x))
   expect_identical(st_crs(x), st_crs(x))
   expect_identical(x$elev^2, x2$elev2)
-})
\ No newline at end of file
+})
+
+test_that("empty agr attribute is named after subset", {
+	sf = st_sf(data.frame(x = st_sfc(st_point(1:2))))
+	out = sf[, "geometry"]
+	agr = attr(out, "agr")
+	expect_identical(names(agr), character())
+})
